# -*- coding: utf-8 -*-
"""
garage.logger

Logging for debug purposes.

HOW TO USE:

    # set the following in your Django project settings:
    LOG_DIR = os.path.join(SITE_ROOT, 'log')
    LOG_FILE = os.path.join(LOG_DIR, '%s.log' % server_acct)
    LOG_FORMAT = "%(asctime)s - %(name)s - %(levelname)s - %(message)s"
    LOG_PROJECT_CODE = server_acct

    # import "logger" and log debug messages with logger().debug():
    from garage.logger import logger
    logger().debug('this is a debug message')

* created: 2011-03-13 Kevin Chan <kefin@makedostudio.com>
* updated: 2014-11-21 kchan
"""

from __future__ import (absolute_import, unicode_literals)

import logging


# settings for debug log - add to project settings:

# LOG_DIR = os.path.join(SITE_ROOT, 'log')
# LOG_FILE = os.path.join(LOG_DIR, '%s.log' % server_acct)
# LOG_FORMAT = "%(asctime)s - %(name)s - %(levelname)s - %(message)s"
# LOG_PROJECT_CODE = server_acct


# simple logger using Python logging module

log_levels = {
    'notset': logging.NOTSET,
    'debug': logging.DEBUG,
    'info': logging.INFO,
    'warning': logging.WARNING,
    'error': logging.ERROR,
    'critical': logging.CRITICAL
}

log_fmt = "%(asctime)s - %(name)s - %(levelname)s - %(message)s"

def create_log(logname, logfile=None, level='debug', format=log_fmt):
    """
    Create and return simple file logger.

    * level is keyword in `log_levels` (`notset`, `debug`, `info`, etc.)
    * format is format of log entry to output

    :More info: `<http://docs.python.org/library/logging.html>`_

    :param logname: name of log.
    :param logfile: path of log file.
    :param level: log level (see ``log_levels``).
    :param format: log entry format (default is ``log_fmt``).
    :returns: logger object.
    """
    log_level = log_levels.get(level)
    logger = logging.getLogger(logname)
    logger.setLevel(log_level)
    if logfile:
        handler = logging.FileHandler(logfile)
    else:
        handler = logging.StreamHandler()
    handler.setLevel(log_level)
    handler.setFormatter(logging.Formatter(format))
    logger.addHandler(handler)
    return logger


# function to return a logging object for debug and diagnostic use

DebugLogger = None

def logger():
    """
    :return: simple logger object.
    """
    from garage import get_setting
    global DebugLogger
    if not DebugLogger:
        proj = get_setting('LOG_PROJECT_CODE')
        logfile = get_setting('LOG_FILE')
        logfmt = get_setting('LOG_FORMAT')
        DebugLogger = create_log(logname=proj, logfile=logfile, format=logfmt)
    return DebugLogger



# -*- coding: utf-8 -*-
"""
urlgen.py

Django Uri Param Generator

from:
http://djangosnippets.org/snippets/1734/

-----------------------------------
USAGE EXAMPLE

[ urlGen Location ]
    |-- __init__.py
    |-- urlgen.py
[ Sample File views.py ]

from urlgen.urlgen import urlGen
uri = urlGen()
orderURI = uri.generate('order', request.GET)
return render_to_response('templates.html', {'orderURI': orderURI,}, context_instance=RequestContext(request))

[ template.html ]
Order By : <a href="{{ pageURI }}name">Name</a> | <a href="{{ orderURI }}latest">Latest</a>

[ HTML OUTPUT ]
<a href="?order=name">Name</a> | <a href="?order=latest">Latest</a>

uri = urlGen()

order = uri.generate('order', request.GET)
# OUTPUT
# ?order=
page = uri.generate('page', {'search': 'unus', 'order': 'name', 'page': 15})
# OUTPUT
# ?search=unus&order=name&page=
-----------------------------------

* created: 2011-01-07 Kevin Chan <kefin@makedostudio.com>
* updated: 2014-11-21 kchan
"""


class urlGen:

    """
        REVISION: 1.0
        AUTHOR: NICKOLAS WHITING

        Build a URL QuerString based on given uri, param
        to add current querystring for a URL

        param: Parameter to add to querystring

        uri: Dictionary of current querystring Params | Accepts django's request.GET

        Usage Example

        Add a current URI param and remove if it exists

        uri = urlGen()

        Current URI: ?page=1&search=nick
        uri.urlgen('page', request.GET)
        Outputs: ?search=nick&page=

        Add a uri Param and exclude a current

        Current URI: ?search=nick&page=2
        urlgen('order', request.GET, ['page',])
        Outputs: ?search=nick&order=


    """

    def generate(self, param, uri = {}, exclude = []):
        self.param = param
        self.uri = uri
        self.exclude = exclude

        self.querystring = False

        """
        BUG FIX:

        Append param to exclude to ensure the param
        Doesnt get added twice to URI

        """
        exclude.append(param)

        # Add the URI Param if it is the only one given

        if len(self.uri) == 0:
            try:
                self.appendQuerystring(self.param)
            except ExceptionError:
                raise ExceptionError (
                    'urlgen recieved an unexpected error adding %s param failed' % (params)
                    )
        else:
            for k,v in self.uri.iteritems():
                if self.param is not str(k) and k not in self.exclude:
                    self.appendQuerystring(k, v)

            # Append the param to end of URL

            self.appendQuerystring(self.param)

        return self.querystring

    # original version:
    #
    # def appendQuerystring(self, param, value = False):
    #
    #     """
    #     Appends a param to the current querystring
    #     """
    #     if self.querystring is False:
    #         if value is False:
    #             self.querystring = '?%s=' % (str(param))
    #         else:
    #             self.querystring = '?%s=%s' % (str(param), str(value))
    #     else:
    #         if value is False:
    #             self.querystring  = '%s&%s=' % (self.querystring, str(param))
    #         else:
    #             self.querystring  = '%s&%s=%s' % (self.querystring, str(param), str(value))
    
    def appendQuerystring(self, param, value=False):

        """
        Appends a param to the current querystring
        """
        if self.querystring is False:
            if value is False:
                self.querystring = '?%s=' % (param)
            else:
                self.querystring = '?%s=%s' % (param, value)
        else:
            if value is False:
                self.querystring  = '%s&%s=' % (self.querystring, param)
            else:
                self.querystring  = '%s&%s=%s' % (self.querystring, param, value)



# -*- coding: utf-8 -*-
"""
garage.html_utils

Utility functions to handle html-text conversions.

* FIXME: need to reorganize, refactor and delete unnecessary code.

* created: 2008-06-22 kevin chan <kefin@makedostudio.com>
* updated: 2014-11-21 kchan
"""

from __future__ import (absolute_import, unicode_literals)

import six
import re
from htmlentitydefs import codepoint2name, name2codepoint
from markdown import markdown
from textile import textile


# functions to escape html special characters

def html_escape(text):
    """
    Escape reserved html characters within text.
    """
    htmlchars = {
        "&": "&amp;",
        '"': "&quot;",
        "'": "&apos;",
        ">": "&gt;",
        "<": "&lt;",
    }
    if isinstance(text, six.text_type):
        text = ''.join([htmlchars.get(c, c) for c in text])
    return text


def html_entities(u):
    """
    Convert non-ascii characters to old-school html entities.
    """
    result = []
    for c in u:
        if ord(c) < 128:
            result.append(c)
        else:
            try:
                result.append('&%s;' % codepoint2name[ord(c)])
            except KeyError:
                result.append("&#%s;" % ord(c))
    return ''.join(result)


def escape(txt):
    """
    Escapes html reserved characters (<>'"&) and convert non-ascii
    text to html entities.
    * To escape only html reserved characters (<>'"&), use
      `html_escape`.
    """
    return html_escape(html_entities(txt))


def unescape(text):
    """
    Removes HTML or XML character references and entities from a text string.
    * Note: does not strip html tags (use `strip_tags` instead for that).

    :Info: http://effbot.org/zone/re-sub.htm#unescape-html

    :param text: The HTML (or XML) source text.
    :return: The plain text, as a Unicode string, if necessary.
    """
    def fixup(m):
        text = m.group(0)
        if text[:2] == "&#":
            # character reference
            try:
                if text[:3] == "&#x":
                    return unichr(int(text[3:-1], 16))
                else:
                    return unichr(int(text[2:-1]))
            except ValueError:
                pass
        else:
            # named entity
            try:
                text = unichr(name2codepoint[text[1:-1]])
            except KeyError:
                pass
        return text # leave as is
    return re.sub("&#?\w+;", fixup, text)


def strip_tags(html_txt):
    """
    Strip tags from html text (uses strip_tags from django.utils.html).
    * also unescapes html entities
    * fall back on using `re.sub` if django's `strip_tags` is not
      importable for some reason.
    """
    try:
        from django.utils.html import strip_tags as _strip_tags
    except ImportError:
        stripped = re.sub(r'<[^>]*?>', '', html_txt)
    else:
        stripped = _strip_tags(html_txt)
    return unescape(stripped)



# functions for converting plain text content to html
# * available conversion methods:
#   * no conversion
#   * markdown
#   * textile
#   * simple conversion of line breaks
#   * visual editor (using wysiwyg editor like TinyMCE)

NO_CONVERSION = 1
MARKDOWN_CONVERSION = 2
TEXTILE_CONVERSION = 3
SIMPLE_CONVERSION = 4
VISUAL_EDITOR = 5

CONVERSION_CHOICES = (
    (NO_CONVERSION, 'None'),
    (MARKDOWN_CONVERSION, 'Markdown'),
    (TEXTILE_CONVERSION, 'Textile'),
    (SIMPLE_CONVERSION, 'Simple (Convert Line Breaks)'),
    (VISUAL_EDITOR, 'Visual (WYSIWYG) Editor'),
)

CONVERSION_METHODS = (
    (NO_CONVERSION, 'none'),
    (MARKDOWN_CONVERSION, 'markdown'),
    (TEXTILE_CONVERSION, 'textile'),
    (SIMPLE_CONVERSION, 'markdown'),
    (VISUAL_EDITOR, 'visual')
)

def txt2html(txt, method):
    try:
        assert txt is not None and len(txt) > 0
        if method == MARKDOWN_CONVERSION:
            txt = markdown(txt)
        elif method == TEXTILE_CONVERSION:
            txt = textile(txt)
        elif method == SIMPLE_CONVERSION:
            txt = markdown(txt)
        else:
            # NO_CONVERSION
            pass
    except (TypeError, AssertionError):
        pass
    return txt


def get_cvt_method(name):
    """
    Get conversion method "code" corresponding to name
    """
    c = {
        'none': NO_CONVERSION,
        'markdown': MARKDOWN_CONVERSION,
        'textile': TEXTILE_CONVERSION
    }
    try:
        method = c.get(name.lower(), NO_CONVERSION)
    except (TypeError, AttributeError):
        method = NO_CONVERSION
    return method


def get_cvt_method_name(code):
    """
    Get conversion method name corresponding to "code"
    """
    if code > 0:
        code -= 1
    try:
        codenum, name = CONVERSION_METHODS[code]
    except:
        codenum, name = CONVERSION_METHODS[NO_CONVERSION]
    return name


def to_html(txt, cvt_method='markdown'):
    """
    Convert text block to html
    * cvt_method is name of method (markdown, textile, or none)
    * cf. txt2html where method is the conversion "code" (number)
    """
    return txt2html(txt, get_cvt_method(cvt_method))


# html-to-text utility function

def html_to_text(html):
    """
    Utility function to convert html content to plain text.
    * uses Django `strip_tags` function to convert html to text;
    * multiple blank lines are reduced to 2;
    * strips beginning and ending white space.
    * does not perform any kind of formatting or structuring to the
      plain text result.

    :param html: html content
    :returns: plain text content after conversion
    """
    from garage.text_utils import tidy_txt
    txt = tidy_txt(strip_tags(html))
    lines = []
    for line in txt.splitlines():
        s = line.strip()
        if len(s) > 0:
            lines.append(s)
    txt = '\n\n'.join(lines)
    txt = '%s\n' % txt
    return txt



# -*- coding: utf-8 -*-
"""
garage.cache

Cache helpers
* The cache functions/decorators use the Django's caching backend.

* created: 2011-03-14 Kevin Chan <kefin@makedostudio.com>
* updated: 2015-02-22 kchan
"""

from __future__ import (absolute_import, unicode_literals)

import six
import functools
import hashlib
try:
    import cPickle as pickle
except ImportError:
    import pickle

from django.core.cache import cache


def s2hex(s):
    """
    Convert any string to a hex-digit hash for use as cache key.
    * uses MD5 hash from hashlib

    :param s: string to hash
    :returns: hash of string as hex digits
    """
    try:
        hashed = hashlib.md5(s).hexdigest()
    except UnicodeEncodeError:
        hashed = hashlib.md5(s.encode('utf-8')).hexdigest()
    except TypeError:
        hashed = hashlib.md5(pickle.dumps(s)).hexdigest()
    return hashed


CACHE_KEY_SEPARATOR = '/'

def cache_key(keystr, *args, **kwargs):
    """
    A helper function to calculate a hashed cache key.
    * accepts the following keyword parameters:
      key_separator -- separator string to use when concatenating args
      prefix -- prefix to prepend to key (after hashing)

    :param keystr: text to create key hash
    :param args: list of strings to join to "keystr"
    :param kwargs: keyword arguments (see code below for keywords)
    :returns: hashed key
    """
    key_separator = kwargs.get('key_separator', CACHE_KEY_SEPARATOR)
    prefix = kwargs.get('prefix')

    if not hasattr(keystr, '__iter__'):
        key_data = [keystr]
    else:
        key_data = keystr
    if len(args) > 0:
        key_data.extend(args)

    elems = []
    for s in key_data:
        if not isinstance(s, six.string_types):
            s = pickle.dumps(s)
        elems.append(s)

    key_string = key_separator.join(elems)
    key = s2hex(key_string)
    if prefix is not None:
        key = '{0}{1}'.format(prefix, key)
    return key

def create_cache_key(name, *args, **kwargs):
    """
    Same as cache_key (for compatibility with legacy function).
    """
    return cache_key(name, *args, **kwargs)


DEFAULT_TIMEOUT = 1800

def cache_data(key=None, timeout=DEFAULT_TIMEOUT):
    """
    A decorator to cache objects.
    * see: http://djangosnippets.org/snippets/492/

    How to use:

    # cache output of ``myfunc`` for 5 min.
    @cache_data(key='mymodule.myfunc', timeout=300)
    def myfunc(arg):
        ...

    # to delete/invalidate cached data, use the ``delete_cache``
    # function:
    delete_cache('mymodule.myfunc')

    """
    def decorator(f):
        @functools.wraps(f)
        def _cache_controller(*args, **kwargs):
            if key is None:
                k = f.__name__
            elif isinstance(key, six.string_types):
                k = key
            elif hasattr(key, '__call__'):
                k = key(*args, **kwargs)
            result = cache.get(k)
            if result is None:
                result = f(*args, **kwargs)
                cache.set(k, result, timeout)
            return result
        return _cache_controller
    return decorator


def delete_cache(key):
    """
    Delete cached object.

    :param key: key to retrieve data from cache
    :returns: True if cached data is found and deleted else False
    """
    if key in cache:
        cache.set(key, None, 0)
        return True
    else:
        return False



# -*- coding: utf-8 -*-
"""
garage.db

Helper functions for managing Django models and fetching querysets.

* created: 2011-02-15 Kevin Chan <kefin@makedostudio.com>
* updated: 2015-02-22 kchan
"""

from __future__ import (absolute_import, unicode_literals)

import copy


# batch queryset iterator
#
# from:
# http://djangosnippets.org/snippets/1170/
#
# Most of the time when I need to iterate over Whatever.objects.all()
# in a shell script, my machine promptly reminds me that sometimes even
# 4GB isn't enough memory to prevent swapping like a mad man, and
# bringing my laptop to a crawl. I've written 10 bazillion versions of
# this code. Never again.
#
# Caveats
#
# Note that you'll want to order the queryset, as ordering is not
# guaranteed by the database and you might end up iterating over some
# items twice, and some not at all. Also, if your database is being
# written to in between the time you start and finish your script,
# you might miss some items or process them twice.
#

# batch size for queryset iterator
# * number of entries to retrieve and iterate in batches
DEFAULT_QS_BATCH_SIZE = 50

def batch_qs(qs, batch_size=DEFAULT_QS_BATCH_SIZE):
    """
    Returns a (start, end, total, queryset) tuple for each batch in the given
    queryset.

    Usage:

    # Make sure to order your queryset before using this function
    article_qs = Article.objects.order_by('id')
    for start, end, total, qs in batch_qs(article_qs):
        print "Now processing %s - %s of %s" % (start + 1, end, total)
        for article in qs:
            print article.body

    """
    total = qs.count()
    for start in range(0, total, batch_size):
        end = min(start + batch_size, total)
        yield (start, end, total, qs[start:end])


class ClonableMixin(object):
    """
    Model mixin with method to clone objects.
    * from:
      http://djangosnippets.org/snippets/1271/
    """

    def clone(self):
        """Return an identical copy of the instance with a new ID."""
        if not self.pk:
            raise ValueError('Instance must be saved before it can be cloned.')
        duplicate = copy.copy(self)
        # Setting pk to None tricks Django into thinking this is a new object.
        duplicate.pk = None
        duplicate.save()
        # ... but the trick loses all ManyToMany relations.
        for field in self._meta.many_to_many:
            source = getattr(self, field.attname)
            destination = getattr(duplicate, field.attname)
            for item in source.all():
                destination.add(item)
        return duplicate


def clone_objects(objects):
    """
    Generic model object cloner function.
    * see:
      http://www.bromer.eu/2009/05/23/a-generic-copyclone-action-for-django-11/
      http://djangosnippets.org/snippets/1271/
    * The function below combines code from the above reference articles.

    :param objects: model instances to clone
    :returns: new objects list
    """
    def clone(obj):
        """Return an identical copy of the instance with a new ID."""
        if not obj.pk:
            raise ValueError('Instance must be saved before it can be cloned.')
        duplicate = copy.copy(obj)
        # Setting pk to None tricks Django into thinking this is a new object.
        duplicate.pk = None
        duplicate.save()
        # ... but the trick loses all ManyToMany relations.
        for field in obj._meta.many_to_many:
            source = getattr(obj, field.attname)
            destination = getattr(duplicate, field.attname)
            for item in source.all():
                destination.add(item)
        return duplicate

    if not hasattr(objects,'__iter__'):
        objects = [objects]

    objs = []
    for obj in objects:
        new_obj = clone(obj)
        new_obj.save()
        objs.append(new_obj)

    return objs



# -*- coding: utf-8 -*-
"""
garage.utils

General utility functions.

* created: 2008-08-11 kevin chan <kefin@makedostudio.com>
* updated: 2015-02-22 kchan
"""

from __future__ import (absolute_import, unicode_literals)

import six
import os
import sys
import re
import hashlib
import base64
import codecs

try:
    import cPickle as pickle
except ImportError:
    import pickle

import yaml

try:
    from yaml import (
        CLoader as Loader,
        CDumper as Dumper
    )
except ImportError:
    from yaml import Loader, Dumper


def get_instance(module, class_name, *args, **kwargs):
    """
    Return an instance of the object based on
    module name and class name.

    :param module: module name (e.g. garage.utils)
    :param class_name: name of class defined in module (e.g. DataObject)
    :param args, kwargs: args and kwargs to supply to class when
        creating instance
    :returns: object instance of class
    """
    if not module in sys.modules:
        __import__(module)
    f = getattr(sys.modules[module], class_name)
    obj = f(*args, **kwargs)
    return obj


# file read/write/delete helper functions

default_text_encoding = "utf-8"
default_encoding = default_text_encoding

def open_file(path, mode=None, encoding=None, **kwargs):
    """
    Helper function to open a file for reading/writing.

    :param path: path of file to read.
    :param mode: "b" for bytes or "t" for text (default is "t")
    :param encoding: file encoding for text (default is `utf-8`).
    :returns: stream object for reading/writing
    """
    try:
        from io import open as _open
    except ImportError:
        def _open(path, mode=None, encoding=None, **kwargs):
            return codecs.open(path, mode, encoding=encoding, **kwargs)
    return _open(path, mode=mode, encoding=encoding, **kwargs)


def get_file_contents(path, mode=None, encoding=None, **kwargs):
    """
    Load text file from file system and return content as text.

    :param path: path of file to read.
    :param mode: "b" for bytes or "t" for text (default is "t")
    :param encoding: file encoding for text (default is `utf-8`).
    :returns: file content as string or `None` if file cannot be read.

    This function reads the entire content of the file before
    returning the data as a string or as bytes.
    """
    try:
        assert path is not None and os.path.isfile(path)
    except AssertionError:
        data = None
    else:
        if not mode:
            mode = ''
        mode = 'r%s' % mode
        if 'b' in mode:
            encoding = None
        else:
            # read file as text
            if not encoding:
                encoding = default_text_encoding
        with open_file(path, mode=mode, encoding=encoding, **kwargs) as file_obj:
            data = file_obj.read()
    return data


def write_file(path, data, mode=None, encoding=None, **kwargs):
    """
    Write text file to file system.

    :param path: path of file to write to.
    :param data: data to write.
    :param mode: "b" for bytes or "t" for text (default is "t")
    :param encoding: file encoding for text (default is `utf-8`).
    :returns: `True` if no error or `False` if ``IOError``.
    """
    if not mode:
        mode = ''
    mode = 'w%s' % mode
    if 'b' in mode:
        encoding = None
    else:
        if not encoding:
            encoding = default_text_encoding
    try:
        with open_file(path, mode=mode, encoding=encoding, **kwargs) as file_obj:
            file_obj.write(data)
        return True
    except IOError:
        return False


def delete_file(path):
    """
    Truncates file to zero size and unlinks file.

    :param path: file system path for file
    :returns: True if file is unlinked (no longer found) else False
    """
    if os.path.isfile(path) is False:
        # file does not exist
        return True
    with open_file(path, mode='wb') as file_obj:
        file_obj.truncate(0)
    try:
        os.unlink(path)
    except OSError:
        pass
    return os.path.isfile(path) is False


def make_dir(path):
    """
    Make sure path exists by create directories
    * path should be directory path
      (example: /home/veryloopy/www/app/content/articles/archives/)

    :param path: path to directory
    :returns: True if directory exists else False
    """
    if not os.path.exists(path):
        try:
            os.makedirs(path)
        except OSError:
            pass
    if os.path.exists(path):
        return True
    else:
        return False


# YAML utilities

def load_yaml(data):
    """
    Parse yaml data.

    # yaml usage:
    # data = load(stream, Loader=Loader)
    # output = dump(data, Dumper=Dumper)

    :param data: YAML-formatted data
    :returns: loaded data structure
    """
    return yaml.load(data, Loader=Loader)


def load_yaml_docs(data):
    """
    Parse a series of documents embedded in a YAML file.

    * documents are delimited by '---' in the file

    :param data: YAML-formatted data
    :returns: generator object with loaded data
    """
    return yaml.load_all(data, Loader=Loader)


def dump_yaml(data, **opts):
    """
    Dump data structure in yaml format.

    example usage:
    print(dump_yaml(y, explicit_start=True, default_flow_style=False))

    :param data: data structure
    :param opts: optional parameters for yaml engine
    :returns: YAML-formatted `basestring` for output
    """
    return yaml.dump(data, Dumper=Dumper, **opts)


def sha1hash(s):
    """
    Calculate sha1 hash in hex for string.
    """
    try:
        return hashlib.sha1(s).hexdigest()
    except UnicodeEncodeError:
        return hashlib.sha1(s.encode('utf-8')).hexdigest()


# encode/decode functions
# * note: encode_sdata and decode_sdata do not perform any sort of
#   encryption

def encode_sdata(data):
    """
    Encode data (dict) using pickle, b16encode and base64
    * Use ``decode_sdata`` to convert base16-encoded string back into
      Python data.

    :param data: any Python data object
    :returns: pickled byte string of data
    """
    return base64.b16encode(pickle.dumps(data))

def decode_sdata(encoded_string):
    """
    Decode data pickled and encoded using encode_sdata
    * Input must be base16-encoded string produced by the
      ``encode_sdata`` function.

    :param encoded_string: pickled string of data
    :returns: unpickled data or None if error
    """
    try:
        return pickle.loads(base64.b16decode(encoded_string))
    except (TypeError, EOFError, pickle.UnpicklingError):
        return None


class DataObject(dict):
    """
    Data object class
    * based on webpy dict-like Storage object

    # DataObject class for storing generic dict key/value pairs
    # * object is a dict that behaves like a class object (key/value
    #   can be accessed like object attributes).
    #
    # borrowed from web.py
    #
    # class Storage(dict):
    #   '''
    #   A Storage object is like a dictionary except `obj.foo` can be used
    #   in addition to `obj['foo']`.
    #
    #       >>> o = storage(a=1)
    #       >>> o.a
    #       1
    #       >>> o['a']
    #       1
    #       >>> o.a = 2
    #       >>> o['a']
    #       2
    #       >>> del o.a
    #       >>> o.a
    #       Traceback (most recent call last):
    #           ...
    #       AttributeError: 'a'
    #
    #   '''
    #   def __getattr__(self, key):
    #       try:
    #           return self[key]
    #       except KeyError, k:
    #           raise AttributeError, k
    #
    #   def __setattr__(self, key, value):
    #       self[key] = value
    #
    #   def __delattr__(self, key):
    #       try:
    #           del self[key]
    #       except KeyError, k:
    #           raise AttributeError, k
    #
    #   def __repr__(self):
    #       return '<Storage ' + dict.__repr__(self) + '>'
    #
    """

    def __init__(self, *args, **kwargs):
        self.add(*args, **kwargs)

    def __getattr__(self, key):
        try:
            return self[key]
        except KeyError, k:
            raise AttributeError, k

    def __setattr__(self, key, value):
        self[key] = value

    def __delattr__(self, key):
        try:
            del self[key]
        except KeyError, k:
            raise AttributeError, k

    def __repr__(self):
        return '<DataObject ' + dict.__repr__(self) + '>'

    def add(self, *args, **kwargs):
        """
        add({
            'a': 1,
            'b': 3.14
            'c': 'foo'
        })
        """
        for d in args:
            if isinstance(d, six.string_types):
                self[d] = True
            elif isinstance(d, dict):
                for name, value in d.items():
                    self[name] = value
            else:
                try:
                    for name in d:
                        self[name] = True
                except TypeError:
                    pass
        for name, value in kwargs.items():
            try:
                self[name] = value
            except TypeError:
                pass


def enum(*sequential, **named):
    """
    Enum type with ``reverse_mapping`` dict to retrieve key from value.

    from:
    http://stackoverflow.com/a/1695250/220783

    >>> Numbers = enum(ONE=1, TWO=2, THREE='three')
    >>> Numbers.ONE
    1
    >>> Numbers.TWO
    2
    >>> Numbers.THREE
    'three'
    >>> Numbers.reverse_mapping['three']
    'THREE'

    """
    enums = dict(zip(sequential, range(len(sequential))), **named)
    reverse = dict((value, key) for key, value in enums.iteritems())
    enums['reverse_mapping'] = reverse
    return type(str('Enum'), (), enums)


# get file extension

def get_file_ext(filename):
    """
    Extract extension for file.
    * This simple function is here for legacy reasons. Some projects
      import this function from this module so it's here to keep those
      from breaking.

    :param filename: filename or path
    :returns: tuple of file base name and extension (extension is '' if none)
    """
    return os.path.splitext(filename)


# for compatibiity with old versions

def cvt2list(s):
    """Convert object to list"""
    if not hasattr(s, '__iter__'):
        s = [s]
    return s


from garage.text_utils import (
    trim,
    check_eos,
    has_digits,
    has_alpha,
    has_alphanum,
    to_camel_case,
    substitute,
    subs,
    safe_unicode,
    safe_str
)



# -*- coding: utf-8 -*-
"""
garage

Utilities and helpers functions.

* created: 2011-02-15 Kevin Chan <kefin@makedostudio.com>
* updated: 2015-02-23 kchan
"""

from __future__ import (absolute_import, unicode_literals)

import warnings

try:
    from django.core.exceptions import ImproperlyConfigured
    from django.conf import settings as _django_settings
except (ImportError, ImproperlyConfigured):
    from garage.utils import DataObject
    _django_settings = DataObject()
    warnings.warn('Unable to import Django settings! Please check your setup '
                  'and make sure Django is installed and your project settings '
                  'are loaded correctly.', RuntimeWarning)



# package version
__author__ = 'Kevin Chan'
__version_info__ = (0, 1, 12)
__version__ = '.'.join(map(str, __version_info__))

VERSION = __version_info__

def get_version(version=None):
    """
    Returns a PEP 440 version string (in the form of major.minor.micro).
    Http://legacy.python.org/dev/peps/pep-0440/
    """
    if version is None:
        version = __version_info__
    if isinstance(version, (list, tuple)):
        version = '.'.join(map(str, version))
    return version


# helper functions and shortcuts

# get setting

def get_setting(name, default=None):
    """Retrieve attribute from settings."""
    return getattr(_django_settings, name, default)


# legacy functions for compatibility with old code using this package

def resp(request, template, context):
    """Shortcut for render_to_response()."""
    from django.shortcuts import render_to_response
    from django.template import RequestContext
    return render_to_response(template, context,
                              context_instance=RequestContext(request))


# for compatibility with old code that imports the following from here
from garage.session import set_session_var, get_session_var



"""
setup.py

Setup.py for django-garage.

* created: 2013-01-12 Kevin Chan <kefin@makedostudio.com>
* updated: 2014-11-21 kchan
"""

import os
from setuptools import setup

README = open(os.path.join(os.path.dirname(__file__), 'README.md')).read()

# allow setup.py to be run from any path
os.chdir(os.path.normpath(os.path.join(os.path.abspath(__file__), os.pardir)))


setup(
    name = "django-garage",
    version = "0.1.12",
    packages = ['garage'],
    include_package_data = True,
    license = "BSD",
    description = "A collection of useful functions and modules for Django development",
    long_description = README,
    url = "https://bitbucket.org/kchan/django-garage",
    author = "Kevin Chan",
    author_email = "kefin@makedostudio.com",
    classifiers = [
        'Development Status :: 4 - Beta',
        'Environment :: Web Environment',
        'Framework :: Django',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: BSD License',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Programming Language :: Python :: 2.6',
        'Programming Language :: Python :: 2.7',
        'Topic :: Internet :: WWW/HTTP',
        'Topic :: Internet :: WWW/HTTP :: Dynamic Content',
    ],
    install_requires = [
        'six',
        'django',
        'markdown',
        'textile',
        'Pillow',
        'pyyaml',
        'pytz',
        'mock',
    ]
)
